package models

import (
	"eta/eta_index_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"strconv"
	"strings"
	"time"
)

// BaseFromCCF CCF化纤信息
type BaseFromCCF struct{}

type BaseFromCCFData struct {
	BaseFromCcfDataId  int `orm:"column(base_from_ccf_data_id);pk"`
	BaseFromCcfIndexId int
	IndexCode          string
	DataTime           string
	Value              string
	CreateTime         time.Time
	ModifyTime         time.Time
	DataTimestamp      int64
}

func (m *BaseFromCCFData) TableName() string {
	return "base_from_ccf_data"
}

func GetBaseFromCCFDataByCondition(condition string, pars []interface{}) (list []*BaseFromCCFData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_ccf_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

// Add 添加
func (obj BaseFromCCF) Add(edbCode string) (err error) {
	o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code = ? "
		pars = append(pars, edbCode)
	}
	ccfBaseDataAll, err := GetBaseFromCCFDataByCondition(condition, pars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_ccf(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range ccfBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

// Refresh 刷新CCF化纤指标数据
func (obj BaseFromCCF) Refresh(edbInfoId int, edbCode, startDate string) (err error) {
	source := obj.GetSource()
	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	ccfDataList, err := GetBaseFromCCFDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, 0, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_ccf(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range ccfDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, 0, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, 0, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromBaiinfo add Err", err.Error())
			return
		}
	}
	return
}

// GetSource 获取来源编码id
func (obj BaseFromCCF) GetSource() int {
	return utils.DATA_SOURCE_CCF
}

// GetSourceName 获取来源名称
func (obj BaseFromCCF) GetSourceName() string {
	return utils.DATA_SOURCE_NAME_CCF
}

type BaseFromCCFIndex struct {
	BaseFromCcfIndexId int64 `orm:"column(base_from_ccf_index_id);pk"`
	IndexCode          string
	IndexName          string
	Frequency          string
	Unit               string
	ClassifyId         int
	StartDate          string
	EndDate            string
	Sort               int
	TerminalCode       string
	CreateTime         time.Time
	ModifyTime         time.Time
}

func (m *BaseFromCCFIndex) TableName() string {
	return "base_from_ccf_index"
}

func (m *BaseFromCCFIndex) Add() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(m)
	return
}

func (m *BaseFromCCFIndex) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(m, updateCols...)

	return
}

func (m *BaseFromCCFIndex) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_ccf_index SET start_date=?,end_date=?,modify_time=NOW() WHERE index_code=? `
	_, err = o.Raw(sql, item.MinDate, item.MaxDate, indexCode).Exec()
	return
}

func (m *BaseFromCCFIndex) GetByIndexCode(indexCode string) (item *BaseFromCCFIndex, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_ccf_index WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

func (m *BaseFromCCFIndex) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*BaseFromCCFIndex, err error) {
	o := orm.NewOrm()
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := `ORDER BY create_time DESC`
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM base_from_ccf_index WHERE 1=1 %s %s`, fields, condition, order)
	_, err = o.Raw(sql, pars).QueryRows(&items)
	return
}

type BaseFromCCFIndexList struct {
	CcfIndexId   int64 `orm:"column(ccf_index_id);pk"`
	IndexCode    string
	IndexName    string
	Frequency    string
	Unit         string
	ClassifyId   int
	StartDate    string
	EndDate      string
	TerminalCode string
	CreateTime   string
	ModifyTime   string
}

func (m *BaseFromCCFData) GetByIndexCode(indexCode string) (list []*BaseFromCCFData, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_ccf_data WHERE index_code=? `
	_, err = o.Raw(sql, indexCode).QueryRows(&list)
	return
}

func (m *BaseFromCCFData) AddMulti(item []*BaseFromCCFData) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(100, item)
	return
}

// HandleCCFEdbData CCF化纤的指标数据
type HandleCCFEdbData struct {
	IndexName    string            `description:"指标名称"`
	IndexCode    string            `description:"指标编码"`
	ClassifyId   int               `description:"分类ID"`
	Unit         string            `description:"单位"`
	Sort         int               `description:"排序"`
	Frequency    string            `description:"频度"`
	TerminalCode string            `description:"终端编码"`
	DateData     map[string]string `description:"日期数据"`
}

type HandleCCFEdbDataReq struct {
	List         []*HandleCCFEdbData
	TerminalCode string `description:"编码"`
}

type HandleCCFTableData struct {
	ClassifyId   int       `description:"分类ID"`
	FromPage     string    `description:"表格来源"`
	TableDate    time.Time `description:"表格日期"`
	TableContent string    `description:"表格HTML"`
}

type HandleCCFStockTableReq struct {
	Table        *HandleCCFTableData
	TerminalCode string `description:"编码"`
}

func (m *BaseFromCCFData) GetMaxAndMinDateByIndexCode(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_ccf_data WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

// BaseFromCCFClassify CCF化纤数据分类表
type BaseFromCCFClassify struct {
	ClassifyId      int       `orm:"column(classify_id);pk"`
	ClassifyName    string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
	ModifyTime      time.Time `description:"修改时间"`
	CreateTime      time.Time `description:"创建时间"`
}

func (m *BaseFromCCFClassify) Add() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(m)
	return
}

func (m *BaseFromCCFClassify) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(m, updateCols...)

	return
}

func (m *BaseFromCCFClassify) GetByClassifyName(classifyName string) (item *BaseFromCCFClassify, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_ccf_classify WHERE classify_name=? `
	err = o.Raw(sql, classifyName).QueryRow(&item)
	return
}

func (m *BaseFromCCFClassify) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*BaseFromCCFClassify, err error) {
	o := orm.NewOrm()
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := `ORDER BY create_time DESC`
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM base_from_ccf_classify WHERE 1=1 %s %s`, fields, condition, order)
	_, err = o.Raw(sql, pars).QueryRows(&items)
	return
}

// BaseFromCCFClassifyItem CCF化纤数据分类信息
type BaseFromCCFClassifyItem struct {
	ClassifyId   int                        `description:"分类ID"`
	ClassifyName string                     `description:"分类名称"`
	ParentId     int                        `description:"父级id"`
	Level        int                        `description:"层级"`
	Sort         int                        `description:"排序字段"`
	CreateTime   string                     `description:"创建时间"`
	ModifyTime   string                     `description:"修改时间"`
	Child        []*BaseFromCCFClassifyItem `description:"子分类"`
}

func (m *BaseFromCCFClassify) Format2Item(origin *BaseFromCCFClassify) (item *BaseFromCCFClassifyItem) {
	if origin == nil {
		return
	}
	item = new(BaseFromCCFClassifyItem)
	item.ClassifyId = origin.ClassifyId
	item.ClassifyName = origin.ClassifyName
	item.ParentId = origin.ParentId
	item.Level = origin.Level
	item.Sort = origin.Sort
	item.CreateTime = utils.TimeTransferString(utils.FormatDateTime, origin.CreateTime)
	item.ModifyTime = utils.TimeTransferString(utils.FormatDateTime, origin.ModifyTime)
	return
}

// MultiUpdateBaseFromCCFDataValue 批量更新CCF化纤指标数据
func MultiUpdateBaseFromCCFDataValue(items []*BaseFromCCFData) (err error) {
	if len(items) == 0 {
		return
	}

	o := orm.NewOrm()
	sql := `UPDATE base_from_ccf_data SET value = ?, modify_time = NOW() WHERE index_code = ? AND data_time = ? LIMIT 1`
	p, err := o.Raw(sql).Prepare()
	if err != nil {
		return
	}
	defer func() {
		_ = p.Close()
	}()
	for _, v := range items {
		if v.IndexCode == "" || v.DataTime == "" {
			continue
		}
		_, err = p.Exec(v.Value, v.IndexCode, v.DataTime)
		if err != nil {
			return
		}
	}
	return
}
