package models

import (
	"errors"
	"eta/eta_index_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"strconv"
	"strings"
	"time"
)

// SavePredictCalculateLjzzy 累计值转月
func SavePredictCalculateLjzzy(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName, lang string) (edbInfo *EdbInfo, latestDateStr string, latestValue float64, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("SavePredictCalculateLjzzy,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if req.EdbInfoId <= 0 {
		edbInfo = new(EdbInfo)
		edbInfo.EdbInfoType = 1
		edbInfo.Source = utils.DATA_SOURCE_PREDICT_CALCULATE_LJZZY
		edbInfo.SourceName = "预测累计值转月值"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbNameEn = req.EdbName
		edbInfo.UnitEn = req.Unit
		edbInfo.EdbType = 2
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		edbInfo.EdbInfoId = int(newEdbInfoId)

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfo.EdbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfo, err = GetEdbInfoById(req.EdbInfoId)
		if err != nil {
			return
		}
		latestDateStr = edbInfo.LatestDate
		latestValue = edbInfo.LatestValue
		//修改指标信息
		switch lang {
		case utils.EnLangVersion:
			edbInfo.EdbNameEn = req.EdbName
			edbInfo.UnitEn = req.Unit
		default:
			edbInfo.EdbName = req.EdbName
			edbInfo.Unit = req.Unit
			edbInfo.EdbNameSource = req.EdbName
		}
		edbInfo.Frequency = req.Frequency
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.ModifyTime = time.Now()
		_, err = to.Update(edbInfo, "EdbName", "EdbNameSource", "Frequency", "Unit", "ClassifyId", "ModifyTime", "EdbNameEn", "UnitEn")
		if err != nil {
			return
		}

		var existCondition string
		var existPars []interface{}
		existCondition += " AND edb_info_id=? AND from_edb_info_id=? "
		existPars = append(existPars, edbInfo.EdbInfoId, req.FromEdbInfoId)

		var count int
		//判断计算指标是否被更换
		count, err = GetEdbInfoCalculateCountByCondition(existCondition, existPars)
		if err != nil {
			err = errors.New("判断指标是否改变失败,Err:" + err.Error())
			return
		}
		if count > 0 { // 指标未被替换，无需重新计算
			return
		}

		//删除，计算指标关联的，基础指标的关联关系
		sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
		if err != nil {
			return
		}

		//清空原有数据
		sql = ` DELETE FROM edb_data_predict_calculate_ljzzy WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
		if err != nil {
			return
		}

		//关联关系
		{
			calculateMappingItem := &EdbInfoCalculateMapping{
				EdbInfoCalculateMappingId: 0,
				EdbInfoId:                 edbInfo.EdbInfoId,
				Source:                    utils.DATA_SOURCE_PREDICT_CALCULATE_LJZZY,
				SourceName:                "预测累计值转月值",
				EdbCode:                   edbInfo.EdbCode,
				FromEdbInfoId:             fromEdbInfo.EdbInfoId,
				FromEdbCode:               fromEdbInfo.EdbCode,
				FromEdbName:               fromEdbInfo.EdbName,
				FromSource:                fromEdbInfo.Source,
				FromSourceName:            fromEdbInfo.SourceName,
				FromTag:                   "",
				Sort:                      1,
				CreateTime:                time.Now(),
				ModifyTime:                time.Now(),
			}
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	}

	//计算数据
	latestDateStr, latestValue, err = refreshAllPredictCalculateLjzzy(to, edbInfo.EdbInfoId, edbInfo.Source, edbInfo.SubSource, fromEdbInfo, edbInfo.EdbCode, "", "")

	return
}

// RefreshAllPredictCalculateLjzzy 刷新全部累计值转月数据
func RefreshAllPredictCalculateLjzzy(edbInfoId, source, subSource int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (latestDateStr string, latestValue float64, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("refreshAllPredictCalculateLjzzy,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	// 计算数据
	latestDateStr, latestValue, err = refreshAllPredictCalculateLjzzy(to, edbInfoId, source, subSource, fromEdbInfo, edbCode, startDate, endDate)

	return
}

func refreshAllPredictCalculateLjzzy(to orm.TxOrmer, edbInfoId, source, subSource int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (latestDateStr string, latestValue float64, err error) {
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	dataList, err := GetPredictEdbDataListAllByStartDate(fromEdbInfo, 1, startDate)
	if err != nil {
		return
	}
	latestDateStr = fromEdbInfo.LatestDate
	yearMap := make(map[int]map[int]*EdbInfoSearchData)
	dataLen := len(dataList)
	for i := 0; i < dataLen; i++ {
		item := dataList[i]
		//日其中获取年
		var itemDate time.Time
		itemDate, err = time.ParseInLocation(utils.FormatDate, item.DataTime, time.Local)
		if err != nil {
			return
		}
		year := itemDate.Year()
		month := int(itemDate.Month())
		if monthMap, yok := yearMap[year]; yok {
			monthMap[month] = item
			yearMap[year] = monthMap
		} else {
			monthMap = make(map[int]*EdbInfoSearchData)
			monthMap[month] = item
			yearMap[year] = monthMap
		}
	}

	addSql := ` INSERT INTO edb_data_predict_calculate_ljzzy(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	//获取指标所有数据
	existDataList := make([]*EdbData, 0)
	dataTableName := GetEdbDataTableName(source, subSource)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return
	}
	dataMap := make(map[string]string)
	for _, v := range existDataList {
		dataMap[v.DataTime] = v.Value
	}
	existDataMap := make(map[string]string)

	for yk, yv := range yearMap {
		_, oneMonthOk := yv[1]
		_, twoMonthOk := yv[2]
		if !oneMonthOk && !twoMonthOk {
			continue
		}
		for i := 1; i <= 12; i++ {
			fmt.Println(yk, i, yv[i])
			dataCurrentItem := yv[i]
			var date string
			var val float64
			if dataCurrentItem != nil {
				if i == 1 || i == 2 {
					if _, mok := yv[1]; mok { //1月有值
						if i == 1 {
							date = dataCurrentItem.DataTime
							val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Float64() //a.Div(b).Float64()
						}
						if i == 2 {
							dataOneItem := yv[1]
							date = dataCurrentItem.DataTime
							twoMonth := decimal.NewFromFloat(dataCurrentItem.Value)
							oneMonth := decimal.NewFromFloat(dataOneItem.Value)
							val, _ = twoMonth.Sub(oneMonth).Float64()
						}
					} else { //1月无值
						dataTwoItem := yv[2]
						if i == 1 {
							date = strconv.Itoa(yk) + "-01-31"
							a := decimal.NewFromFloat(dataTwoItem.Value)
							b := decimal.NewFromFloat(2.0)
							val, _ = a.Div(b).Float64()
						}
						if i == 2 {
							//1月无值：1月=2月/2
							{
								date = strconv.Itoa(yk) + "-01-31"
								a := decimal.NewFromFloat(dataTwoItem.Value)
								b := decimal.NewFromFloat(2.0)
								val, _ = a.Div(b).Float64()
								tmpSql, newAdd, tmpErr := calculateLjzzy(edbInfoId, date, edbInfoIdStr, edbCode, dataTableName, addSql, val, dataMap, existDataMap, to)
								if !isAdd {
									isAdd = newAdd
								}
								addSql = tmpSql
								if tmpErr != nil {
									err = tmpErr
									return
								}
							}
							//end 1月无值

							date = dataCurrentItem.DataTime
							a := decimal.NewFromFloat(dataTwoItem.Value)
							b := decimal.NewFromFloat(2.0)
							val, _ = a.Div(b).Float64()
						}
					}
				} else {
					dataPreItem := yv[i-1]
					if dataPreItem != nil {
						date = dataCurrentItem.DataTime
						//val =  dataCurrentItem.Value - dataPreItem.Value
						a := decimal.NewFromFloat(dataCurrentItem.Value)
						b := decimal.NewFromFloat(dataPreItem.Value)
						val, _ = a.Sub(b).Float64()
					}
				}
			}
			if date != "" {
				//saveValue := utils.SubFloatToString(val, 4)
				////判断数据是否存在
				//if existVal, ok := dataMap[date]; !ok {
				//	dataTime, _ := time.ParseInLocation(utils.FormatDate, date, time.Local)
				//	timestamp := dataTime.UnixNano() / 1e6
				//	timeStr := fmt.Sprintf("%d", timestamp)
				//	if _, existOk := existDataMap[date]; !existOk {
				//		addSql += GetAddSql(edbInfoIdStr, edbCode, date, timeStr, saveValue)
				//		isAdd = true
				//	}
				//	existDataMap[date] = date
				//} else {
				//	if existVal != saveValue {
				//		sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
				//		sql = fmt.Sprintf(sql, dataTableName)
				//		_, err = to.Raw(sql, saveValue, edbInfoId, date).Exec()
				//		if err != nil {
				//			return err
				//		}
				//	}
				//}
				tmpSql, newAdd, tmpErr := calculateLjzzy(edbInfoId, date, edbInfoIdStr, edbCode, dataTableName, addSql, val, dataMap, existDataMap, to)
				if !isAdd {
					isAdd = newAdd
				}
				addSql = tmpSql
				if tmpErr != nil {
					err = tmpErr
					return
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("refreshAllPredictCalculateLjzzy add Err", err.Error())
			return
		}
	}

	//确定最终值
	var finalLast EdbInfoSearchData
	sql = ` SELECT data_time , value FROM edb_data_predict_calculate_ljzzy WHERE edb_info_id=? and data_time<=? ORDER BY data_time DESC `
	tmpErr := to.Raw(sql, edbInfoId, latestDateStr).QueryRow(&finalLast)
	if tmpErr != nil {
		if tmpErr.Error() != utils.ErrNoRow() {
			err = tmpErr
		}
		return
	} else {
		latestDateStr = finalLast.DataTime
		latestValue = finalLast.Value
	}
	return
}
